unit uImageDefs;

interface
uses
  {$IFDEF FPC}
  LCLType, LCLIntf,
  Graphics,
  {$ELSE}
  WinAPI.Windows,
  Vcl.Graphics,
  {$ENDIF}
  uExifInfoReader;

type
TImageRotate=(tr0,tr90,tr180,tr270);
TImageOperation=(tgs_none,
                 tgs_average,
                 tgs_min,
                 tgs_max,
                 tgs_maxsubmin,
                 tgs_maxsubavarage,
                 tgs_invert,
                 tgs_blackwhite,
                 tgs_normalize,
                 tgs_push_red,
                 tgs_keep_red_only,
                 tgs_remove_blue,
                 tgs_remove_green,
                 tgs_increase_brightness);

TPixelMap=array of array of TRGBTriple;

TSetOfColors=array of TColor;


TDrawType=(tdr_CenterPoint,
           tdr_AreaBox,
           tdr_FocusPoint,
           tdr_CenterRect);

function Bitmap2PixelMap(const Bitmap:TBitmap;const _MakeBlackWhite:boolean;out PixelMap:TPixelMap):boolean;
procedure PixelMap2Bitmap(var Pixels:TPixelMap;var Bitmap:TBitmap);
function GetExifData(const _JPGfilename:string;out Exif:TExif):boolean;
function ExifOrientationToRotate(_Orientation:Byte):TImageRotate;
function TripletoColor(const _RGBTriple: TRGBTriple): TColor;
function IsColorInSet(const _Color:TColor;const _SetOfColors:TSetOfColors):Boolean;

implementation

uses
  {$IFDEF FPC}
  SysUtils,
  Classes,
  Dialogs;
  {$ELSE}
  System.SysUtils,
  System.Classes,
  VCL.Dialogs;
  {$ENDIF}

function IsColorInSet(const _Color:TColor;const _SetOfColors:TSetOfColors):Boolean;
var
i:integer;
begin
  result:=false;
  for i:=0 to length(_SetOfColors)-1 do begin
    if _SetOfColors[i]<>_Color then continue;
    result:=true;
    exit;
  end;
end;

function TripletoColor(const _RGBTriple: TRGBTriple): TColor;
begin
  result := rgb(_RGBTriple.rgbtRed, _RGBTriple.rgbtGreen, _RGBTriple.rgbtBlue);
end;

{-----------------------------------------------------------------------------
  Procedure: Bitmap2PixelMap
  Author:    sam
  Date:      31-Mai-2021
  Arguments: var Bitmap:Vcl.Graphics.TBitmap
  Result:    TPixelMap
  Description: copy the bitmap-pixels to a two dimensional array.
-----------------------------------------------------------------------------}
function Bitmap2PixelMap(const Bitmap:TBitmap;const _MakeBlackWhite:boolean;out PixelMap:TPixelMap):boolean;
var
y,x: integer;
row: ^TRGBTriple;
_Value:byte;
begin
  result:=false;
  if not assigned(Bitmap) then exit;
  SetLength(PixelMap,Bitmap.Height);
  for y:=0 to Bitmap.Height-1 do begin
    SetLength(PixelMap[y],Bitmap.Width);
    row:= Bitmap.Scanline[y];
    for x:=0 to Bitmap.Width-1 do begin
      if _MakeBlackWhite then begin
        if Row^.rgbtred>100 then _Value:=255
                            else _Value:=0;
        if PixelMap[y,x].rgbtred<>   _Value   then PixelMap[y,x].rgbtred  := _Value;
        if PixelMap[y,x].rgbtgreen<> _Value   then PixelMap[y,x].rgbtgreen:= _Value;
        if PixelMap[y,x].rgbtblue <> _Value   then PixelMap[y,x].rgbtblue := _Value;
      end
      else begin
        if PixelMap[y,x].rgbtred<>   Row^.rgbtred   then PixelMap[y,x].rgbtred  := Row^.rgbtred;
        if PixelMap[y,x].rgbtgreen<> Row^.rgbtgreen then PixelMap[y,x].rgbtgreen:= Row^.rgbtgreen;
        if PixelMap[y,x].rgbtblue <> Row^.rgbtblue  then PixelMap[y,x].rgbtblue := Row^.rgbtblue;
      end;
      inc(row);
    end;
  end;
  result:=true;
end;

{-----------------------------------------------------------------------------
  Procedure: PixelMap2Bitmap
  Author:    sam
  Date:      31-Mai-2021
  Arguments: var Pixels:TPixelMap;var Bitmap:Vcl.Graphics.TBitmap
  Result:    None
  Description: copy the two dimensional array into the bitmap.
-----------------------------------------------------------------------------}
procedure PixelMap2Bitmap(var Pixels:TPixelMap;var Bitmap:TBitmap);
var
y,x: integer;
row: ^TRGBTriple;
begin
  for y:= 0 to Bitmap.Height-1 do begin
    row:= Bitmap.Scanline[y];
    for x:= 0 to Bitmap.Width-1 do begin
      if Row^.rgbtred  <>Pixels[y,x].rgbtred   then Row^.rgbtred  :=Pixels[y,x].rgbtred;
      if Row^.rgbtgreen<>Pixels[y,x].rgbtgreen then Row^.rgbtgreen:=Pixels[y,x].rgbtgreen;
      if Row^.rgbtblue <>Pixels[y,x].rgbtblue  then Row^.rgbtblue :=Pixels[y,x].rgbtblue;
      inc(row);
    end;
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: GetExifData
  Author:    sam
  Date:      29-Nov-2021
  Arguments: const _JPGfilename:string;out Exif:TExif
  Result:    boolean
  Description: read exif data from image file <_filename>.
-----------------------------------------------------------------------------}
function GetExifData(const _JPGfilename:string;out Exif:TExif):boolean;
begin
  result:=false;
  Exif:=TExif.Create;
  if not SameText(trim(ExtractFileExt(_JPGfilename)),'.jpg',loUserLocale) then exit;
  if not fileexists(_JPGfilename) then exit;
  try
    if not Exif.ReadFromFile(_JPGfilename) then exit;
    result:=true;
  except
    on e:exception do showmessagefmt('Error in uImageDefs.GetImageOrientation: Could not read orientation of file <%s>. <%s>.',[_JPGfilename,e.message]);
  end;
end;

function ExifOrientationToRotate(_Orientation:Byte):TImageRotate;
begin
  case _Orientation of
    1:result:=tr0;
    8:result:=tr90;
    3:result:=tr180;
    6:result:=tr90;
    else result:=tr0;
  end;
end;

end.
