unit uMisc;

interface

uses
 {$IFDEF FPC}
  LCLType, LCLIntf, FileUtil,
 {$ELSE}
  Windows,
 {$ENDIF}
  Sysutils;

const

  trcError=1;    // errors  (marked red)
  trcWarn =2;    // warnings (marked orange)
  trcWork =3;    // workflow Track user actions.
  trcInOut=5;    // communication port stuff.
  trcCom  =4;    // communication between QCTool and QCConnector/QCTester/XML-Tester
  trcTrace=5;    // trace-level for software developer.
  trcEvent=6;    // show event names. Could be help full to see interaction between the applications.

procedure Trace(const _level:byte;const _msg:String;const _params:array of const);
function CopyFile(const _from,_to:string;const _CreateTargetFolderIfNotExists:boolean=true):boolean;
function CopyFolder(_fromDir, _toDir: string;const _OverwriteIfExists:boolean;const _Silent:boolean=true): Boolean;
function DeleteFolder(_dir: string): Boolean;
function CreateFolder(_dir:string):boolean;
function GetField(const _ch:char;var s:string;const _ReadFromEnd:boolean=false):string;

implementation

uses
  Forms
  {$IFNDEF FPC}
  , ShlObj
  , ShellApi
  {$ENDIF}
  ;

{-----------------------------------------------------------------------------
  Procedure: CopyFolder
  Author:    herzogs2
  Date:      03-Aug-2010
  Arguments: const fromDir, toDir: string;const _OverwriteIfExists:boolean
  Result:    Boolean
  Description: copy a folder.
-----------------------------------------------------------------------------}
function CopyFolder(_fromDir, _toDir: string;const _OverwriteIfExists:boolean;const _Silent:boolean=true): Boolean;
{$IFDEF FPC}
begin
  Result := false;
  // wp: FIXME - implement
end;
{$ELSE}
var
fos: TSHFileOpStruct;
_errorcode:integer;
_errormsg:string;
begin
  result:=false;
  _fromDir:=ExcludeTrailingPathDelimiter(_fromDir);
  _toDir:=IncludeTrailingPathDelimiter(_toDir);
  trace(trcTrace,'CopyFolder: Try to copy from <%s> to <%s>.',[_fromDir,_toDir]);
  if not SysUtils.DirectoryExists(_fromDir) then begin
    trace(trcWarn,'Warning in CopyFolder: Could not find/access source directory <%s>.',[_fromDir]);
    exit;
  end;
  if not SysUtils.DirectoryExists(_toDir) then begin
    if not CreateFolder(_toDir) then begin
      trace(trcWarn,'Warning in CopyFolder: Could not find/access target directory <%s>.',[_toDir]);
      exit;
    end;
  end;
  ZeroMemory(@fos, SizeOf(fos));
  with fos do begin
    Wnd    := Application.Handle;
    wFunc  := FO_COPY;
    fFlags := FOF_NOCONFIRMMKDIR or FOF_NOCONFIRMATION;
    if _Silent then fFlags:=fFlags or FOF_SILENT;
    pFrom  := PChar(_fromDir+#0);
    pTo    := PChar(_toDir+#0);
  end;
  _errorcode:=ShFileOperation(fos);
  Result := (0 = _errorcode);
  if not result then begin
    _errormsg:=SysErrorMessage(GetLastError);
    trace(trcError,'Error in CopyFolder: <%s>. Error code <%d>.',[_errormsg,_errorcode]);
    exit;
  end;
  trace(trcTrace,'CopyFolder: Successfully copied directory <%s> to <%s>.',[_fromDir,_toDir]);
end;
{$ENDIF}

{-----------------------------------------------------------------------------
  Procedure: DeleteFolder
  Author:    herzogs2
  Date:      03-Aug-2010
  Arguments: dir: string
  Result:    Boolean
  Description:
-----------------------------------------------------------------------------}
function DeleteFolder(_dir: string): Boolean;
{$IFDEF FPC}
begin
  Result := false;
  // wp: FIXME: implement
end;
{$ELSE}
var
fos: TSHFileOpStruct;
_errormsg:string;
_errorcode:integer;
begin
  result:=false;
  if _dir='' then begin
    trace(trcError,'Error in DeleteFolder: Input parameter <_Dir> is empty.',[]);
    exit;
  end;
  _dir:=ExcludeTrailingPathDelimiter(_dir); // remove trailing pathdelimiters.
  trace(trcTrace,'DeleteFolder:Try to delete directory <%s>.',[_dir]);
  if not SysUtils.DirectoryExists(_dir) then begin
    result:=true;
    trace(trcTrace,'DeleteFolder: The directory <%s> does not exist. Nothing to delete.',[_dir]);
    exit;
  end;
  ZeroMemory(@fos, SizeOf(fos));
  with fos do
  begin
    Wnd    := Application.Handle;
    wFunc  := FO_DELETE;
    fFlags := FOF_SILENT or FOF_NOCONFIRMATION;
    pFrom  := PChar(_dir+#0);
  end;
  _errorcode:=ShFileOperation(fos);
  result := (0 = _errorcode);
  if not result then begin
    _errormsg:=SysErrorMessage(GetLastError);
    trace(trcError,'Error in DeleteFolder: <%s>. Error code <%d>.',[_errormsg,_errorcode]);
    exit;
  end;
  trace(trcTrace,'DeleteFolder: Successfully deleted directory <%s>.',[_dir]);
end;
{$ENDIF}

{-----------------------------------------------------------------------------
  Procedure: CreateFolder
  Author:    herzogs2
  Date:      03-Aug-2010
  Arguments: const _dir:string
  Result:    boolean
  Description:
-----------------------------------------------------------------------------}
function CreateFolder(_dir:string):boolean;
begin
  result:=false;
  _dir:=ExcludeTrailingPathDelimiter(_dir);   // remove trailing pathdelimiters.
  trace(trcTrace,'CreateFolder: Try to create directory <%s>.',[_dir]);
  if SysUtils.DirectoryExists(_dir) then begin
    trace(trcTrace,'The directory <%s> exists already. Leave method. Nothing to do.',[_dir]);
    result:=true;
    exit;
  end;
  try
    result:=SysUtils.ForceDirectories(_dir);
    if not result then begin
      trace(trcError,'CreateFolder: Could not create directory <%s>.',[_dir]);
      exit;
    end;
    trace(trcTrace,'Created directory <%s>.',[_dir]);
  except
    on e:exception do trace(trcError,'Error in CreateFolder: Could not create directory <%s>. <%s>.',[_dir,e.message]);
  end;
end;


{-----------------------------------------------------------------------------
  Procedure: Trace
  Author:    Not available
  Date:      28-Aug-2002
  Arguments: _msg:String;_params:array of const
  Result:    None
  Description: Write the to the debug log.
-----------------------------------------------------------------------------}
procedure Trace(const _level:byte;const _msg:String;const _params:array of const);
{$ifdef debugtrace}
var
  _s:string;
{$endif}
begin
  {$ifdef debugtrace}
  if SizeOf(_params)>0 then _s:=format(_msg,_params)
                       else _s:=_msg;
  OutputDebugString(PChar(_s));
  {$endif}
end;

{-----------------------------------------------------------------------------
  Procedure: CopyFile
  Author:    herzogs2
  Date:      07-Sep-2009
  Arguments: const _from,_to:string;const _CreateTargetFolderIfNotExists:boolean=true
  Result:    boolean
  Description: copy a file from <_from> to <_to> and create the target folder if <_CreateTargetFolderIfNotExists> set to true.
-----------------------------------------------------------------------------}
function CopyFile(const _from,_to:string;const _CreateTargetFolderIfNotExists:boolean=true):boolean;
var
_TargetFolder:string;
begin
  result:=false;
  if not fileexists(_from) then begin
    trace(trcWarn,'Warning in CopyFile: Can not copy file <%s> because it does not exist.',[_from]);
    exit;
  end;
  _TargetFolder:=IncludeTrailingPathDelimiter(ExtractFilePath(_to));
  if (not SysUtils.DirectoryExists(_TargetFolder)) and
     (_CreateTargetFolderIfNotExists) then begin
    try
      SysUtils.ForceDirectories(_TargetFolder);
    except
      on e:exception do trace(trcError,'Error in CopyFile: Could not create folder <%s> because of <%s>.',[_TargetFolder,e.message]);
    end;
  end;
  if not SysUtils.DirectoryExists(_TargetFolder) then begin
    trace(trcWarn,'Warning in CopyFile: Can not copy file <%s> to folder <%s> because the folder does not exist.',[_from,_TargetFolder]);
    exit;
  end;
  try
   {$IFDEF FPC}
    Result := FileUtil.CopyFile(_from, _to);
   {$ELSE}
    result:=windows.CopyFile(pchar(_from),pchar(_to),false);
   {$ENDIF}
    if not result then
      {$IFDEF FPC}
      trace(trcError,'Problem in CopyFile: Can not copy file <%s> to <%s>.',[_from,_to]);
      {$ELSE}
      trace(trcError,'Problem in CopyFile: Can not copy file <%s> to <%s>. <%s>.',[_from,_to,SysErrorMessage(GetLastError)]);
      {$ENDIF}
  except
    on e:exception do trace(trcError,'Error in CopyFile: Can not copy file <%s> to <%s>. <%s>.',[_from,_to,e.message]);
  end;
end;


function GetField(const _ch:char;var s:string;const _ReadFromEnd:boolean=false):string;
var
_pos:integer;
begin
  result:='';
  _pos:=Pos(_ch,s);
  if _pos=0 then begin
    result:=s;
    s:='';
    exit;
  end;
  if _ReadFromEnd then _pos:=LastDelimiter(_ch,s);
  result:=Copy(s,1,_pos-1);
  Delete(s,1,_pos);
end;

end.




